--[[
Author: Michael Joseph Murray aka Lyte of Lothar(US)
Prescription is a consumables usage check addon designed for end-game raiding
$Date: 2010-05-28 22:21:38 +0000 (Fri, 28 May 2010) $
$Revision: 137 $
r138
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2007-2010 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

Prescription = LibStub("AceAddon-3.0"):NewAddon("Prescription", "AceTimer-3.0", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Prescription")

local fmt = string.format
local tinsert = table.insert
local tremove = table.remove
local find = string.find
local pairs = pairs
local strlower = string.lower

local battleList = {
	--	wotlk elixirs	--
	L["Accuracy"], --Elixir of Accuracy
	L["Armor Piercing"], --Elixir of Armor Piercing
	L["Deadly Strikes"], --Elixir of Deadly Strikes
	L["Expertise"], --Elixir of Expertise
	L["Lightning Speed"], -- Elixir of Lightning Speed
	L["Mighty Agility"], -- Elixir of Mighty Agility
	L["Mighty Strength"], --Elixir of Mighty Strength
	L["Guru's Elixir"],
	L["Spellpower Elixir"],
	L["Wrath Elixir"],
}

local guardianList = {
	--	wotlk elixirs	--
	L["Mighty Defense"], --Elixir of Mighty Defense
	L["Elixir of Mighty Fortitude"],
	L["Mighty Mana Regeneration"], --Elixir of Mighty Mageblood
	L["Mighty Thoughts"], --Elixir of Mighty Thoughts
	L["Protection"], -- Elixir of Mighty Protection
	L["Elixir of Spirit"],
}

local flaskList = {
	--	wotlk flasks		--
	L["Flask of Endless Rage"],
	L["Flask of Pure Mojo"],
	L["Flask of Stoneblood"],
	L["Flask of the Frost Wyrm"],
	--	Patch 3.2		--
	L["Flask of the North"],
}

local foodList = {
	L["Well Fed"],
}

local user = UnitName("player")
local commTable = {}

function Prescription:OnInitialize()
	local defaults = {
		profile = {
			checkBattle = true,
			checkGuardian = true,
			checkFood = true,
			checkFlask = true,
			sendWhispers = false,
			sendRaid = false,
			sendOfficer = false,
			sendSelf = true,
			blockOutgoing = false,
			delay = 15,
			sendCompliance = true,
			sendChannel = false,
			channel = "",
			hideAlert = false,
		},
	}
	
	self.db = LibStub("AceDB-3.0"):New("PrescriptionDB", defaults, "Default")
	defaults = nil
	
	local opts = {
		type = 'group',
		args = {
			check = {
				type = 'execute',
				name = L["Check Consumables"],
				desc = L["Check the raid for consumable usage"],
				func = function()
					self:ScheduleTimer("Raid_Check", self.db.profile.delay)
					if not self.db.profile.hideAlert then
						if IsRaidLeader() or IsRaidOfficer() then
							SendChatMessage(fmt(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID_WARNING")
						else
							SendChatMessage(fmt(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID")
						end
					end
				end,
			},
			delay = {
				type = 'range',
				name = L["RX Check Delay"],
				desc = L["Set the delay before a Prescription check is run."],
				get = function() return self.db.profile.delay end,
				set = function(info, v) self.db.profile.delay = v end,
				min = 0,
				max = 60,
				step = 1,
			},
			drugs = {
				type = 'group',
				name = L["Prescription Drugs"],
				desc = L["What do you want to check for?"],
				args = {
					battle = {
						type = 'toggle',
						name = L["Battle Elixirs"],
						desc = L["Toggle checking for Battle Elixirs"],
						get = function() return self.db.profile.checkBattle end,
						set = function() self.db.profile.checkBattle = not self.db.profile.checkBattle end,
					},
					guardian = {
						type = 'toggle',
						name = L["Guardian Elixirs"],
						desc = L["Toggle checking for Guardian Elixirs"],
						get = function() return self.db.profile.checkGuardian end,
						set = function() self.db.profile.checkGuardian = not self.db.profile.checkGuardian end,
					},
					flask = {
						type = 'toggle',
						name = L["Flasks"],
						desc = L["Toggle checking for Flasks"],
						get = function() return self.db.profile.checkFlask end,
						set = function() self.db.profile.checkFlask = not self.db.profile.checkFlask end,
					},
					food = {
						type = 'toggle',
						name = L["Food Buffs"],
						desc = L["Toggle checking for Food Buffs"],
						get = function() return self.db.profile.checkFood end,
						set = function() self.db.profile.checkFood = not self.db.profile.checkFood end,
					},
				},
			},
			announce = {
				type = 'group',
				name = L["Announcements"],
				desc = L["How to announce Prescriptions"],
				args = {
					self = {
						type = 'toggle',
						name = L["Self Report"],
						desc = L["Print consumable report for yourself"],
						get = function() return self.db.profile.sendSelf end,
						set = function() self.db.profile.sendSelf = not self.db.profile.sendSelf end,
						order = 1,
					},
					whisper = {
						type = 'toggle',
						name = L["Whisper Players"],
						desc = L["Whisper players missing consumables"],
						get = function() return self.db.profile.sendWhispers end,
						set = function() self.db.profile.sendWhispers = not self.db.profile.sendWhispers end,
						order = 2,
					},
					blockouts = {
						type = 'toggle',
						name = L["Block Outgoing Whispers"],
						desc = L["Toggles blocking of outgoing <Rx> whispers"],
						get = function() return self.db.profile.blockOutgoing end,
						set = function() self.db.profile.blockOutgoing = not self.db.profile.blockOutgoing end,
						order = 3,
					},
					raid = {
						type = 'toggle',
						name = L["Raid Report"],
						desc = L["Send consumable reports to raid chat"],
						get = function() return self.db.profile.sendRaid end,
						set = function() self.db.profile.sendRaid = not self.db.profile.sendRaid end,
						order = 4,
					},
					officer = {
						type = 'toggle',
						name = L["Officer Report"],
						desc = L["Send consumable reports to officer chat"],
						get = function() return self.db.profile.sendOfficer end,
						set = function() self.db.profile.sendOfficer = not self.db.profile.sendOfficer end,
						order = 5,
					},
					channel = {
						type = 'toggle',
						name = L["Custom Channel Report"],
						desc = L["Send consumable reports to a user specified channel"],
						get = function() return self.db.profile.sendChannel end,
						set = function() self.db.profile.sendChannel = not self.db.profile.sendChannel end,
						order = 6,
					},
					channelName = {
						type = 'input',
						name = L["Custom Channel Name"],
						desc = L["Enter the name of the channel."],
						get = function() return self.db.profile.channel end,
						set = function(_, v) self.db.profile.channel = v end,
						order = 7,
					},
					compliance = {
						type = 'toggle',
						name = L["Compliance Broadcast"],
						desc = L["Broadcast the percent compliance to the raid after running a check."],
						get = function() return self.db.profile.sendCompliance end,
						set = function() self.db.profile.sendCompliance = not self.db.profile.sendCompliance end,
						order = 8,
					},
					hide = {
						type = 'toggle',
						name = L["Hide \"Time to Check\" Alert Message"],
						desc = L["Hide the message that alerts the raid to an incoming RX Check"],
						get = function() return self.db.profile.hideAlert end,
						set = function() self.db.profile.hideAlert = not self.db.profile.hideAlert end,
						order = 9,
					},
				},
			},
		},
	}
	opts.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	
	--register the config
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Prescription", opts)
	LibStub("AceConfigDialog-3.0"):SetDefaultSize("Prescription", 500, 550)
	
	--slashy slashy
	_G["SlashCmdList"]["PRESCRIPTION_MAIN"] = function(s)
		if strlower(s) == "check" then
			self:ScheduleTimer("Raid_Check", self.db.profile.delay)
			if not self.db.profile.hideAlert then
				if IsRaidLeader() or IsRaidOfficer() then
					SendChatMessage(fmt(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID_WARNING")
				else
					SendChatMessage(fmt(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID")
				end
			end
		elseif strlower(s) == "instant" then
			if not self.db.profile.hideAlert then
				if IsRaidLeader() or IsRaidOfficer() then
					SendChatMessage(L["RX Check running now!"], "RAID_WARNING")
				else
					SendChatMessage(L["RX Check running now!"], "RAID")
				end
			end
			self:Raid_Check()
		else
			LibStub("AceConfigDialog-3.0"):Open("Prescription")
		end
	end
	_G["SlashCmdList"]["PRESCRIPTION_CHECK"] = function()
		self:ScheduleTimer("Raid_Check", self.db.profile.delay)
		if not self.db.profile.hideAlert then
			if IsRaidLeader() or IsRaidOfficer() then
				SendChatMessage(fmt(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID_WARNING")
			else
				SendChatMessage(fmt(L["RX Check will be run in %d seconds!"], self.db.profile.delay), "RAID")
			end
		end
	end
	_G["SLASH_PRESCRIPTION_MAIN1"] = "/rx"
	_G["SLASH_PRESCRIPTION_MAIN2"] = "/prescription"
	_G["SLASH_PRESCRIPTION_CHECK1"] = "/rxc"
	
	--message filter for blocking outgoing RX whisper reports
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", function(self, event, ...)
		local msg = ...
		if Prescription.db.profile.blockOutgoing and msg:find("^<RX> ") and UnitInRaid("player") then
			return true, ...
		end
		return false, ...
	end)
end

function Prescription:Raid_Check()
	if GetNumRaidMembers() == 0 then
		self:Print(fmt("|cffff0000%s|r", L["You are not in a raid!"]))
		return
	end
	if InCombatLockdown() then
		self:Print(fmt("|cffff0000%s|r", L["Combat produces irregular results, aborting check."]))
		return
	end
	
	local raid = {}
	local location = GetRealZoneText()
	local compliant = 0
	local processed = 0
	for i = 1, GetNumRaidMembers() do
		--if the unit is online and near the raid group (within 100 yrds) cache the raid member's buffs
		if UnitIsConnected("raid"..i) and UnitIsVisible("raid"..i) and (not UnitIsDead("raid"..i)) then
			local n = 1
			if UnitBuff("raid"..i, n) == nil then
				if not raid[i] then raid[i] = {} end
				tinsert(raid[i], "No Buffs")
			end
			while UnitBuff("raid"..i, n) ~= nil do
				local name = UnitBuff("raid"..i, n)
				if not raid[i] then raid[i] = {} end
				tinsert(raid[i], name)
				n = n + 1
			end
		end
	end
	
	for i = 1, GetNumRaidMembers() do
		local e1, e2, f1, f2 = false, false, false, false
		local toon, _ = UnitName("raid"..i)
		--if the unit is online and near the raid group (within 100 yrds) check the raid member's buffs for consumables
		--this prevents false reports for offline raiders and raiders not in the instance with the raid
		if UnitIsConnected("raid"..i) and UnitIsVisible("raid"..i) and (not UnitIsDead("raid"..i)) then
			processed = processed + 1
			for j = 1, #raid[i] do
				local temp = tremove(raid[i])
				if temp == nil then
					break
				elseif temp == "No Buffs" then
					break
				else
					if self.db.profile.checkBattle then
						for k, v in pairs(battleList) do
							if temp == v then
								e1 = true
							end
						end
					else
						e1 = true
					end
					if self.db.profile.checkGuardian then
						for k, v in pairs(guardianList) do
							if temp == v then
								e2 = true
							end
						end
					else
						e2 = true
					end
					if self.db.profile.checkFlask then
						for k, v in pairs(flaskList) do
							if temp == v then
								f1 = true
							end
						end
					else
						f1 = false
					end
					if self.db.profile.checkFood then
						for k, v in pairs(foodList) do
							if temp == v then
								f2 = true
							end
						end
					else
						f2 = true
					end
				end
			end
			
			--Local message stubs
			local raid = fmt(L["%s is missing: "], toon)
			local whisper = L["You are missing: "]
			
			-- Flask fix for whispering missing buffs
			--if they have a flask, by definition they have battle & guardian
			if f1 then
				e1, e2 = true, true
			end
			
			local noMessage = false
			--Logic for whispering missing buffs
			if not e1 then
				whisper = whisper .. L["Battle Elixir"] .. ", "
				raid = raid .. L["Battle Elixir"] .. ", "
			end
			if not e2 then
				whisper = whisper .. L["Guardian Elixir"] .. ", "
				raid = raid .. L["Guardian Elixir"] .. ", "
			end
			--catch missing flask
			if (not f1) and (not e1) and (not e2) then
				whisper = whisper .. L["Flask"] .. ", "
				raid = raid .. L["Flask"] .. ", "
			end
			--catch missing flask if not checking for either elixir
			if (not f1) and (not self.db.profile.checkBattle or not self.db.profile.checkGuardian) then
				whisper = whisper .. L["Flask"] .. ", "
				raid = raid .. L["Flask"] .. ", "
			end
			if not f2 then
				whisper = whisper .. L["Food"] .. ", "
				raid = raid .. L["Food"] .. ", "
			end
			
			if whisper == L["You are missing: "] then
				noMessage = true
				compliant = compliant + 1
			end
			
			if not commTable[toon] then commTable[toon] = {} end
			if not noMessage then
				tinsert(commTable[toon], whisper)
				tinsert(commTable[toon], raid)
			end
			if noMessage then
				tinsert(commTable[toon], "No Message")
				tinsert(commTable[toon], "No Message")
			end
		end
	end

	--for each key in the commTable, the value is a numerically indexed table upto 2
	--index 1 refers to the message to be whispered to the raiders
	--index 2 refers to the message to be sent to raid/officer chat/self report
	for k in pairs(commTable) do
		for i, v in pairs(commTable[k]) do
			if i == 1 then
				if self.db.profile.sendWhispers and (k ~= user) then
					if v ~= "No Message" then
						SendChatMessage("<RX> "..v, "WHISPER", nil, k)
					end
				end
			end
			if i == 2 then
				if self.db.profile.sendRaid then
					if v ~= "No Message" then
						SendChatMessage("<RX> "..v, "RAID")
					end
				end
				if self.db.profile.sendOfficer then
					if v ~= "No Message" then
						SendChatMessage("<RX> "..v, "OFFICER")
					end
				end
				if self.db.profile.sendSelf then
					if v ~= "No Message" then
						self:Print(v)
					end
				end
				if self.db.profile.sendChannel then
					if v ~= "No Message" then
						SendChatMessage("<RX> "..v, "CHANNEL", nil, GetChannelName(self.db.profile.channel))
					end
				end
			end
		end
		
	end
	
	--print a compliance percent if enabled
	if self.db.profile.sendCompliance then
		if compliant == processed then
			SendChatMessage("<RX> " .. L["100% Compliance, Thank You."], "RAID")
		else
			SendChatMessage("<RX> " .. fmt(L["%d%% Compliance."], compliant / processed * 100), "RAID")
		end
	end
	
	--cleanup tables and flags
	wipe(commTable)
end
